%fine_beam_tube
% e/m for an electron is inferred using measurements taken using a Fine
% Beam Tube. Winchester College, Lab P5. 7th February 2017.
%
% Andy French 

function fine_beam_tube

%Fundamental constants

%Charge on electron /C
qe = 1.602e-19;

%Mass of electron /kg
me = 9.109e-31;

%Charge to mass ratio
qe_over_me_actual = qe/me;

%%

%Equipment parameters

%Radius of Helmholtz coils /m
R = 0.15;

%Distance of Helmholtz coils from centre of tube /m
h = 0.075;

%Number of Helmholtz coil turns
N = 130;

%Fudge factor(!) to scale accelerating voltage. There might be some loss
%e.g. internal resistance which will reduce the actuall accelerating
%voltage from what was measured across the power supply terminals
V_fudge_factor = 0.57;
V_fudge_factor = 0.595;
%V_fudge_factor = 1;

%%

%Graph options

%Fontsize
fsize = 18;

%%

%Measurements

%Load Excel sheet with experimental data
[num,txt,raw] = xlsread('Fine Beam 7-Feb-2017.xlsx');

%Assign vector elements to data variables
for n=1:14
    
    %Helmholtz coil current I /amps
    I(n) = raw{ 6+n, 2 };
    
    %Error in I /amps
    Ierror(n) = 0.005; 
    
    %Accelerating voltage /volts
    V(n) = raw{ 6+n, 8 };
    
    %Accelerating voltage error /volts
    Verror(n) = 0.02;
    
    %Electron beam radius /m
    % This is the mean average of the maximuma and minimum beam radii
    r(n) = raw{ 6+n, 11 };
    
    %Electron beam radius error /m
    % This is set to be half the difference between the maximum and minimum
    % beam radii
    rerror(n) = raw{ 6+n, 12 };
    
    %Calculate magnetic field strength /T
    [Bx,By,Bz,B0] = Bcoil( N, I(n), R, h, r(n) );
    B(n) = Bx;
    [Bx,By,Bz,B0] = Bcoil( N, I(n) - Ierror(n), R, h, r(n) + rerror(n) );
    Bmin(n) = Bx;
    [Bx,By,Bz,B0] = Bcoil( N, I(n) + Ierror(n), R, h, r(n) - rerror(n) );
    Bmax(n) = Bx;
end

%Apply fudge factor to V
V = V_fudge_factor*V;

%Form x and y variables which predicts an equation y = (e/m)*x
x = B.^2;
y = 2*V./(r.^2);

%Determine error bars i.e. range of possible x and y values
xmin = Bmin.^2;
xmax = Bmax.^2;
ymin = 2*(V-Verror)./( (r + rerror).^2 );
ymax = 2*(V+Verror)./( (r - rerror).^2 );

%Form predicted rekationship using actual e/m value
y_predict = qe_over_me_actual*x;

%Find the line of bets fit through the measured data
[ qe_over_m,c,xfit,yfit,r ] = bestfit( x,y,500);

%Plot data
figure('name','fine beam tube e/m','color',[1 1 1])
plot(x,y_predict,'b-'); hold on;
plot(x,y,'r.'); 

%Overlay line of best fit
plot(xfit,yfit,'k-'); 

%Overlay error bars
for n=1:length(x)
    plot( [xmin(n),xmax(n)],[y(n),y(n)] ,'r-' );
    plot( [x(n),x(n)],[ymin(n),ymax(n)] ,'r-' );
    plot( [xmin(n),xmin(n)],[ymin(n),ymax(n)] ,'r-' );
    plot( [xmax(n),xmax(n)],[ymin(n),ymax(n)] ,'r-' );
    plot( [xmin(n),xmax(n)],[ymin(n),ymin(n)] ,'r-' );
    plot( [xmin(n),xmax(n)],[ymax(n),ymax(n)] ,'r-' );
end
legend({'Prediction','Data'},'fontsize',fsize);
xlabel('B^2','fontsize',fsize)
ylabel('2V/r^2','fontsize',fsize)
title({['Fine beam tube experiment. Actual electron e/m = ',num2str(qe_over_me_actual,3)],...
    ['e/m = ',num2str(qe_over_m,3),'. Best fit line r = ',num2str(r,2)]},...
    'fontsize',fsize);
grid on
set(gca,'fontsize',fsize);
print(gcf,'fine beam e_over_m.png','-dpng','-r300');
close(gcf);

%%

%Calculate magnetic field within Helmholtz coils.
% Assumes a pair of coils, N turns per coil. Current I, radius R.
% Axial distance h from each coil, and distance r off axis.
function [Bx,By,Bz,B0] = Bcoil( N, I, R, h, r )

%Permeability of free space /Hm^-1
mu0 = 4*pi*1e-7;

%Discrete angle change for circular loop
P = 1001;
dt = 2*pi/(P-1);

for n=1:numel(r)
    %Define vector from coil origin to B field evaluation point
    rr = [h(n),0,r(n)];
    
    %Calculate Magnetic field using Biot-Savart law
    B = [0,0,0];
    for m=1:P
        t = dt*(m-1);
        dl = R*dt*[ 0, -sin(t), cos(t)];
        dr = rr - [0,R*cos(t),R*sin(t)];
        abs_dr = sqrt( dot(dr,dr) );
        B = B + cross( dl,dr )/ (abs_dr^3) ;
    end
    B = (2*mu0*I*N/(4*pi))*B;
    Bx(n) = B(1);
    By(n) = B(2);
    Bz(n) = B(3);
    
    %Calculate on axis field strength /Tesla
    B0(n) = mu0*N*I/R;
    B0(n) = B0(n) * ( 1 + (h(n)/R)^2 )^(-1.5) ;
end

%%

%bestfit
% Function which determines line of best fit to x,y data
%
% [ m,c,xfit,yfit,r ] = bestfit( x,y,N )
%
% x,y vectors defining x,y data inputs
% N number of data points for fitted line of best fit
%
% xfit,yfit vectors defining coordinates of line of best fit
% m,c parameters of line of best fit of form y = m*x + c
% r is the product moment correlation coefficient.

function [ m,c,xfit,yfit,r ] = bestfit( x,y,N )

%Calculate mean parameters
xbar = mean(x);
ybar = mean(y);
x2bar = mean(x.^2);
y2bar = mean(y.^2);
xybar = mean(x.*y);

%Compute variances
Vx = x2bar - xbar^2;
Vy = y2bar - ybar^2;

%Compute covariance
cov_xy = xybar - xbar*ybar;

%Product moment correlation coefficient
r = cov_xy/sqrt( Vx*Vy );

%Determine parameters of line of best fit y = m*x + c 
m = cov_xy/Vx;
c = ybar - m*xbar;

%Define line of best fit
xfit = linspace( min(x), max(x), N );
yfit = m * xfit + c;

%End of code