%fine_beam_tube_model
% Mathematical model of a Fine Beam Tube
%
% LAST UPDATED by Andy French February 2017

function fine_beam_tube_model

%Fundamental constants

%Charge on electron /C
qe = 1.602e-19;

%Mass of electron /kg
me = 9.109e-31;

%Charge to mass ratio
qe_over_me_actual = qe/me;

%%

%Equipment parameters

%Radius of Helmholtz coils /m
R = 0.15;

%Distance of Helmholtz coils from centre of tube /m
h = 0.075;

%Number of Helmholtz coil turns
N = 130;

%%

%Measurements

%Applied voltage /volts
V = 172.1;

%Coil current /amps
I = 0.965;

%Radius of electron beam /m
r = 0.0444;

%%

%Calculate on axis magnetic field strength

%Calculate qe/m from measurements
[Bx,By,Bz,B0] = Bcoil( N, I, R, h, r );
B = Bx;
qe_over_me = 2*V/(r^2 * B^2 );

%Display results
disp(' ')
disp(' Fine Beam Tube Experiment ')
disp([' Actual qe/me = ',num2str(qe_over_me_actual,4),' C/kg']);
disp([' Measured qe/me = ',num2str(qe_over_me,4),' C/kg']);
disp([' Percentage error = ',num2str( abs( 100*( qe_over_me_actual - qe_over_me )/qe_over_me_actual ), 3 ),'%' ] );
disp([' Accelerating P.D V = ',num2str(V),' volts'])
disp([' Beam radius r = ',num2str(r),' metres'])
disp([' Bx field at r = ',num2str(B*1e3),' milli-Tesla'])
disp(' ');

%Plot how magnetic field in Helmholtz coils varies with off axis distance
%rr /cm
fsize = 18;
rr = linspace(0,20,100);
[Bx,By,Bz,B0] = Bcoil( N, I, R, h*ones(1,100), rr/100);
plot( rr, Bx./B0, rr,By./B0, rr,Bz./B0 );
legend({'Bx/B0','By/B0','Bz/B0'},'fontsize',fsize)
xlabel('Off axis distance /cm','fontsize',fsize)
ylabel('Magnetic field / on axis field strength','fontsize',fsize)
title( 'Off axis magnetic field / on axis field strength','fontsize',fsize );
set(gca,'fontsize',fsize )
grid on;
print(gcf,'off axis B field.png','-dpng','-r300' );

%%

%Calculate magnetic field within Helmholtz coils.
% Assumes a pair of coils, N turns per coil. Current I, radius R.
% Axial distance h from each coil, and distance r off axis.
function [Bx,By,Bz,B0] = Bcoil( N, I, R, h, r )

%Permeability of free space /Hm^-1
mu0 = 4*pi*1e-7;

%Discrete angle change for circular loop
P = 1001;
dt = 2*pi/(P-1);

for n=1:numel(r)
    %Define vector from coil origin to B field evaluation point
    rr = [h(n),0,r(n)];
    
    %Calculate Magnetic field using Biot-Savart law
    B = [0,0,0];
    for m=1:P
        t = dt*(m-1);
        dl = R*dt*[ 0, -sin(t), cos(t)];
        dr = rr - [0,R*cos(t),R*sin(t)];
        abs_dr = sqrt( dot(dr,dr) );
        B = B + cross( dl,dr )/ (abs_dr^3) ;
    end
    B = (2*mu0*I*N/(4*pi))*B;
    Bx(n) = B(1);
    By(n) = B(2);
    Bz(n) = B(3);
    
    %Calculate on axis field strength /Tesla
    B0(n) = mu0*N*I/R;
    B0(n) = B0(n) * ( 1 + (h(n)/R)^2 )^(-1.5) ;
end

%End of code